class Segment {

	public PVector offset;
	public PVector axis;
	public PVector control;
	
	public PVector armLength;

	public PVector storedOffset;
	public PVector storedAxis;
	public PVector storedControl;

	public int seed;

	public float angle;

	public int totalLines = 50;

	public PVector[] orbit = new PVector[totalLines];

	public PVector startRelativeVec;
	public PVector endRelativeVec;
	public PVector startCrossRelativeVec;
	public PVector endCrossRelativeVec;
	public PVector pseudoRelativeVec;

	public PVector startAbsoluteVec;
	public PVector endAbsoluteVec;
	public PVector startCrossAbsoluteVec;
	public PVector endCrossAbsoluteVec;
	public PVector pseudoAbsoluteVec;

		
	color from = color(255, 0, 102);
	color to = color(0, 102, 153);

	public Segment() {
		int radius = 25;

		axis 	= new PVector( 25, 0, 0 );
		offset 	= new PVector( 0,0,0);
		control = new PVector( 0, 10, 0 );

		armLength 	= new PVector( 25, 0, 0 );
		
		storedAxis = new PVector( 0, random(25), random(25) );
		storedOffset = offset.get();
		storedControl = control.get();
		

		createRelatives();
		resetRelatives();
		createAbsolutes();
	}

	public void createRelatives(){
		int radius = 100;
		float half_radius = 100*.5;
		startRelativeVec = new PVector( 0, 0, 0 );
		endRelativeVec = new PVector( random(radius), random(radius)-half_radius, random(radius)-half_radius );
		pseudoRelativeVec = new PVector( 0, 0, 30 );
		startCrossAbsoluteVec = new PVector( 0, 0, 0 );
		endCrossAbsoluteVec = new PVector( 0, 0, 0 );
	}

	public void resetRelatives(){
		
		startCrossRelativeVec = endRelativeVec.cross(pseudoRelativeVec);
		startCrossRelativeVec.normalize();
		startCrossRelativeVec.mult(30);

		endCrossRelativeVec = startCrossRelativeVec.get();
		endCrossRelativeVec.add(endRelativeVec);

	}
	
	
	public void createAbsolutes(){
		startAbsoluteVec = startRelativeVec.get();
		endAbsoluteVec = endRelativeVec.get();
		startCrossAbsoluteVec = startCrossRelativeVec.get();
		endCrossAbsoluteVec = endCrossRelativeVec.get();
		pseudoAbsoluteVec = pseudoRelativeVec.get();
	}
	
	public void updateAbsolutes(PVector offset){
		startAbsoluteVec.add(offset);
		endAbsoluteVec.add(offset);
		startCrossAbsoluteVec.add(offset);
		endCrossAbsoluteVec.add(offset);
		pseudoAbsoluteVec.add(offset);
	}
	
	public void parse() {
		for(int i=0;i<totalLines;i++) {
			// angle = 360*float(i)/float(totalLines) + frameCount*(totalLines-seed+1)*.125;
			angle = 360*float(i)/float(totalLines) +360*(1-float(seed)/float(totalLines)) + frameCount;
			orbit[i] = rotateAroundArbitraryAxis( offset, control, axis, angle );
		}
	}

	public void draw() {
		drawWidthArray(orbit,#FFFFFF);
	}
	
	public void drawGuides() {
		/*
		drawPoint(offset,5,#FF00FF);
		drawPoint(control,3,#00FF00);
		
		drawLine(axis,offset,1,#FFFFFF);
		drawLine(control,offset,1,#FFFF00);
		*/
		
		drawPoint(endAbsoluteVec,3,#FFFFFF);
		drawLine(startAbsoluteVec,endAbsoluteVec,1,#FFFFFF);

		drawPoint(startCrossAbsoluteVec,3,#FF00FF);
		drawLine(startAbsoluteVec,startCrossAbsoluteVec,1,#FF00FF);

		drawPoint(endCrossAbsoluteVec,3,#00FF00);
		drawLine(endAbsoluteVec,endCrossAbsoluteVec,1,#00FF00);

		drawLine(startAbsoluteVec,pseudoAbsoluteVec,1,#00FF00);

	}

	void drawWidthArray(PVector[] orbiter, color colorParam) {

		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		float percentage;
		float colorShifter = sin( ((float(seed) / float(totalSegments) + frameCount*.001) * TWO_PI))*.5+.5;
		strokeWeight(1);
		color interColor = lerpColor(from, to,  colorShifter);

		for(int i=1;i<lines;i++) {
			curr = orbiter[i].get();
			prev = orbiter[i-1].get();

			percentage = (float(i)/float(lines)) * 255;

			noStroke();

			fill(interColor, percentage);
			beginShape(TRIANGLE_FAN);
			vertex(prev.x,prev.y,prev.z);
			vertex(curr.x,curr.y,curr.z);
			vertex(offset.x,offset.y,offset.z);
			vertex(prev.x,prev.y,prev.z);
			endShape();

		}
	}

	public void drawPoint(PVector pointParam, int strokeParam, color colorParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,255);
		point( pointParam.x,pointParam.y,pointParam.z);
	}

	public void drawLine(PVector pointAParam,PVector pointBParam, int strokeParam, color colorParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,102);
		line( pointAParam.x,pointAParam.y,pointAParam.z,pointBParam.x,pointBParam.y,pointBParam.z);
	}
}

